import { w3cwebsocket as WebsocketClient } from "websocket";

type ServiceOptions = {
  name: string;
  websocketUrl: string;
};

export default abstract class Service {
  /**
   * Service options
   */
  private name: string;
  private websocketUrl: string;

  /**
   * The websocket client that is connected to this service.
   */
  private websocket: WebsocketClient;

  constructor({ name, websocketUrl }: ServiceOptions) {
    this.name = name;
    this.websocketUrl = websocketUrl;

    this.websocket = connectWebsocket(this.websocketUrl);

    this.websocket.onopen = () => {
      this.log(
        `Connected to the ${this.name} websocket on ${this.websocketUrl}`
      );
    };

    this.websocket.onclose = () => {
      this.websocket = connectWebsocket(this.websocketUrl);
    };
    this.websocket.onerror = (error: any) => {
      this.log(`An error has occured on websocket ${this.name}: ${error}`);
    };

    this.websocket.onmessage = (message: any) => {
      try {
        const json = JSON.parse(message.data.toString());

        this.message(json);
      } catch (e) {}
    };

    /**
     * Connects to the websocket.
     *
     * @returns the websocket
     */
    function connectWebsocket(websocketUrl: string) {
      return new WebsocketClient(websocketUrl);
    }
  }

  /**
   * Gets called when a message from
   * the websocket is received.
   *
   * @param data the message
   */
  abstract message(data: any): void;

  /**
   * Logs a message to console with
   * the service name.
   *
   * @param message the message to log
   */
  public log(message: string): void {
    console.log(`[${this.name}] ${message}`);
  }

  /**
   * Returns the name of the leaderboard.
   *
   * @returns the name
   */
  public getName() {
    return this.name;
  }

  /**
   * Returns the websocket url of the leaderboard.
   *
   * @returns the websocket url
   */
  public getWebsocketUrl() {
    return this.websocketUrl;
  }
}
